# File with a couple of standard sh-functions for use in i-Package
# phases

# ii2checksanity
# Check sanity of environment variables

sub ii2checksanity
{
    if ($ENV{"II2INSTALLDIR"} eq '') {
	die "### ${PROGNAME}: II2INSTALLDIR is empty. Bailing out...\n";
    }
    if ($ENV{"II2PKGDIR"} eq '') {
	die "### ${PROGNAME}: II2PKGDIR is empty. Bailing out...\n";
    }
}

# ii2calledfrom()
# Report caller if this has been set through ${II2CALLEDFROM}

sub ii2calledfrom()
{
    my $II2CALLEDFROM = $ENV{"II2CALLEDFROM"};
    if ($II2CALLEDFROM ne '') {
	print "### ${PROGNAME}: Called from $II2CALLEDFROM\n";
    }
}

# ii2checksubdirs()
# - first argument: root directory (normally II2INSTALLDIR)
# - rest arguments: subdirs of root directory
# These are checked for existence and writability and warnings are produced

sub ii2checksubdirs
{
    my $rootdir = shift;
    my @subdirs = @_;

    foreach my $subdir (@subdirs) {
	if (not -d "${rootdir}/${subdir}") {
	    print "### ${PROGNAME}: Cannot find ${rootdir}/${subdir}\n";
	    print "### ${PROGNAME}: This is normally no cause for alarm as this is true on initial installs, when uninstall is run more than once, or when the i-Package contents have changed layout over time.\n";
	}
	elsif (not -w "${rootdir}/${subdir}") {
	    die "### ${PROGNAME}: Cannot write in ${rootdir}/${subdir}. Bailing out...";
	}
    }
}

# ii2removeemptysubdirs
# Remove named empty subdirs of rootdir (normally II2INSTALDIR)

sub ii2removeemptysubdirs
{
    my $rootdir = shift;
    my @subdirs = @_;

    foreach my $subdir (@subdirs) {
	print "### ${PROGNAME}: Removing ${rootdir}/${subdir} (if empty)\n";
	system( "rmdir -p \"$rootdir/$subdir\"");
    }
}

# ii2removeaccordingtolistfile
# Take the list file, use it to clean up in the remove script

sub uniq
{
    # Input: @list
    my @list = shift;
    # Output: @uniqed
    my %seen;
    return grep !$seen{$_}++, @list;
}

sub ii2removeaccordingtolistfile
{
    my $pkgdir = $ENV{"II2PKGDIR"};
    my $installdir = $ENV{"II2INSTALLDIR"};
    my $pkgname = $pkgdir;
    $pkgname =~ s/\.ii2$//;
    $pkgname =~ s/.*[^\/]\///;

    foreach my $listfile (("$pkgdir/$pkgname.l.bz2", "$pkgdir/$pkgname-oldversions.l.bz2")) {
	if ($listfile eq "$pkgdir/$pkgname-oldversions.l.bz2") {
	    print( "Removing any known leftovers from previous versions...\n");
	}
	if (-f $listfile and open FILELIST, "/usr/bin/bzcat \"$listfile\"|") {
	    my @filelist = <FILELIST>;
	    close FILELIST;
	    my %dirhash;
	    chomp @filelist;
	    foreach $file (@filelist) {
		if (-e "$installdir/$file") {
		    print( "unlinking $installdir/$file\n");
		    if (unlink( "$installdir/$file") != 1) {
			warn( "Unable to delete $installdir/$file\n");
		    }
		}
		$file =~ s/\/+[^\/]*$//; # strip /blabla
		while ($file =~ /\//) {
		    $dirhash{$file} = 1;
		    $file =~ s/\/+[^\/]*$//; # strip /blabla at end
		}
	    }
	    foreach $file (reverse sort keys %dirhash) {
		if (-d "$installdir/$file") {
		    if (!rmdir( "$installdir/$file")) {
			print( "Directory $installdir/$file not removed ($!)\n");
		    }
		    else {
			print( "Empty directory $installdir/$file removed\n");
		    }
		}
	    }
	}
	elsif ($listfile eq "$pkgdir/$pkgname.l.bz2") {
	    print "Basic listfile cannot be opened. Cannot use automated remove!\n";
	}
    }
}

# ii2runprog
# Run another prog after an informational message

sub ii2runprog
{
    my $progtorun = shift;
    print "### $0: running $progtorun";
    system( "export II2CALLEDFROM=\"$0\"; \"$1\"");
}

# ii2runphase
# Run another phase of this i-Package
# Phase should be remove, selector, configure or prepare and it should of course exist

sub ii2runphase
{
    my $phasename = shift;
    my $II2RESOURCES = $ENV{"II2RESOURCES"};
    my $II2PKGDIR = $ENV{"II2PKGDIR"};
    my $pkgname = `${II2RESOURCES}/basename "${II2PKGDIR}" .ii2`;
    $phasename = "${II2PKGDIR}/${pkgname}.${phasename}";
    ii2runprog( ${phasename});
}

# ii2repairX11
# Leopard and pre-Leopard use different X11 locations. Stuff compiled against X11 on Leopard
# does not run on pre-Leopard, unless /usr/X11 links to an X11 install 

sub ii2fixPreLeopardX11()
{
    my $darwinmainversion=`/usr/bin/uname -r|/usr/bin/sed 's/\..*//'`;
    chomp( $darwinmainversion);

    if (not -e "/usr/X11" and $darwinmainversion < 9) {
	# Pre-Leopard 
	if (-e "/usr/X11R6") {
	    warn "### Fixing pre-Leopard X11 to work with Leopard-compiled programs\n";
	    system( "ln -s /usr/X11R6 /usr/X11");
	}
    }
}


sub getSetInfo
{
    my $II2LOGLEVEL = $ENV{"II2LOGLEVEL"};
    my $II2DOSERVER = $ENV{"II2DOSERVER"};
    my $II2RESOURCES = $ENV{"II2RESOURCES"};
    my %set = ();
    @allsetinfo = `"${II2RESOURCES}/doalerter" -s "${II2DOSERVER}" -V -t GetSelectorSets`;
    if (not defined @allsetinfo) {
	warn "### ${PROGNAME}: Unable to get selector set from package\n";
	exit( 1);
    }
    # warn @allsetinfo;

    $numberofsets = scalar( @allsetinfo) / 7;
    if ($numberofsets >= 1) {
	for (my $i = 0; $i < $numberofsets; $i++) {
	    my $id = $allsetinfo[7*$i];
	    my $name = $allsetinfo[7*$i+1];
	    my $comment = $allsetinfo[7*$i+2];
	    my $selected = $allsetinfo[7*$i+3];
	    my $configured = $allsetinfo[7*$i+4];
	    my $installed = $allsetinfo[7*$i+5];
	    my $available  = $allsetinfo[7*$i+6];

	    chomp( $id);
	    chomp( $name);
	    chomp( $comment);
	    chomp( $selected);
	    chomp( $configured);
	    chomp( $installed);
	    chomp( $available);

	    ${set}{$id}{'name'} = $name;
	    ${set}{$id}{'comment'} = $comment;
	    ${set}{$id}{'selected'} = $selected;
	    ${set}{$id}{'configured'} = $configured;
	    ${set}{$id}{'installed'} = $installed;
	    ${set}{$id}{'available'} = $available;

	    if (${II2LOGLEVEL} > 3) {
		my $msg = "### ${PROGNAME}: Set \"$id\": \"$name\" is";
		$msg .= ($available eq 'YES' ? ' available,' : ' not available,');
		$msg .= ($selected eq 'YES' ? ' selected,' : ' not selected,');
		$msg .= ($installed eq 'YES' ? ' installed and' : ' uninstalled and');
		$msg .= ($configured eq 'YES' ? ' configured.' : ' unconfigured.');
		$msg .= "\n";
		warn $msg;
	    }
	    if (${II2LOGLEVEL} > 4) {
		my $msg = "### ${PROGNAME}: Set \"$id\" description: \"$comment\"\n";
		warn $msg;
	    }
	}
    }
    else {
	warn "### ${PROGNAME}: Empty selector set received from package:\n@allsetinfo\n";
	exit( 1);
    }
    return %set;
}

sub ipkgwarn
{
    my $msg = shift;
    my $II2DOSERVER = $ENV{"II2DOSERVER"};
    warn "### ${PROGNAME}: ${msg}";
    if ($II2DOSERVER ne '') {
	my $II2RESOURCES = $ENV{"II2RESOURCES"};
	my $doresult = `"${II2RESOURCES}/doalerter" -s "${II2DOSERVER}" -V -t Alert -T "Warning/Error." -I "${msg}" -1 "Accept"`;
	chomp( $doresult);
	return $doresult;
    }
    else {
	return 1;
    }
}

sub ipkgsystem
{
    my $cmd = shift;
    my $II2LOGLEVEL = $ENV{"II2LOGLEVEL"};
    if ($II2LOGLEVEL > 3) {
	warn "### ${PROGNAME}. Executing: \"${cmd}\"\n";
    }
    return system( ${cmd});
}

1;
